#ifndef H_PROGRAMS
#define H_PROGRAMS

#if defined(ARDUINO) && ARDUINO >= 100
  #include <Arduino.h>
#else
  #include <WProgram.h>
#endif


#include "program.h"
#include <EEPROM.h>
#include "consts.h"

class Programs{
  public:
    boolean low;
    Programs(){
      low = false;
      currentProgram = 0;
      currentProgramSet = 0;
      currentProgramSet = EEPROM.read(1);
      currentProgram = EEPROM.read(2);

      checkCurrent();      
      setN();     

      for (char i = 0; i<6; i++ ){
        lowBatt.add(0, i, LOW);
      }            
      lowBatt.add(100, 0, LOW);
      for (char i = 0; i<6; i++ ){
        lowBatt.add(0, i, HIGH);
      }            
      lowBatt.add(100, 0, HIGH);


      //********  FIRST  ***********************************************
      for (char i = 0; i<4; i++ ){
        p[0][0].add(0, i, HIGH);
        p[0][0].add(150, i, LOW);
      }            
      for (char i = 0; i<6; i++ ){
        p[1][0].add(0, i, HIGH);
        p[1][0].add(150, i, LOW);
      }            
      //********  SECOND  ***********************************************

      for (char i = 0; i<4; i++ ){
        p[0][1].add(0, i, HIGH);
      }
      for (char i = 0; i<4; i++ ){
        p[0][1].add(0, i, LOW);
        p[0][1].add(200, i, HIGH);
      }            
      
      for (char i = 0; i<6; i++ ){
        p[1][1].add(0, i, HIGH);
      }
      for (char i = 0; i<6; i++ ){
        p[1][1].add(0, i, LOW);
        p[1][1].add(200, i, HIGH);
      }                  
      //********  THIRD  ************************************************
      int d = 100;
     
      p[0][2].add(0, 0, HIGH);
      p[0][2].add(0, 1, HIGH);      
      p[0][2].add(d, 0, LOW);
      p[0][2].add(0, 1, LOW);
      
      p[0][2].add(d, 0, HIGH);
      p[0][2].add(0, 1, HIGH);      
      p[0][2].add(d, 0, LOW);
      p[0][2].add(0, 1, LOW);
      
      p[0][2].add(d+300, 2, HIGH);
      p[0][2].add(0, 3, HIGH);      
      p[0][2].add(d, 2, LOW);
      p[0][2].add(0, 3, LOW);
      
      p[0][2].add(d, 2, HIGH);
      p[0][2].add(0, 3, HIGH);      
      p[0][2].add(d, 2, LOW);
      p[0][2].add(0, 3, LOW);
      
      p[0][2].add(300, 3, LOW);      
      
      //-------------------6      
      
      p[1][2].add(0,5,HIGH);
      p[1][2].add(0,2,HIGH);
      
      p[1][2].add(0, 0, HIGH);
      p[1][2].add(0, 1, HIGH);      
      p[1][2].add(d, 0, LOW);
      p[1][2].add(0, 1, LOW);
      
      p[1][2].add(d, 0, HIGH);
      p[1][2].add(0, 1, HIGH);      
      p[1][2].add(d, 0, LOW);
      p[1][2].add(0, 1, LOW);
      
      p[1][2].add(d+300, 3, HIGH);
      p[1][2].add(0, 4, HIGH);      
      p[1][2].add(d, 3, LOW);
      p[1][2].add(0, 4, LOW);
      
      p[1][2].add(d, 3, HIGH);
      p[1][2].add(0, 4, HIGH);      
      p[1][2].add(d, 3, LOW);
      p[1][2].add(0, 4, LOW);
      
      p[1][2].add(300, 4, LOW);            
      //********  FORTH  ************************************************
      
      for (char i = 0; i<4; i++ ){
        p[0][3].add(0, i, HIGH);
      }            
      p[0][3].add(100,0,HIGH);
      
      for (char i = 0; i<6; i++ ){
        p[1][3].add(0, i, HIGH);
      }            
      p[1][3].add(100,0,HIGH);
      
    }


    
    void start(){
      lowBatt.start();
      p[currentProgramSet][currentProgram].start();      
    }
    void tick(unsigned long dt){
      if (low){
        lowBatt.tick(dt);
      }else{
        p[currentProgramSet][currentProgram].tick(dt);
      }
    }
    
    void incProgram(){
      currentProgram++;
      checkCurrent();
      save();
      start();
    }
    void incProgramSet(){
      currentProgramSet++;
      checkCurrent();
      setN();
      save();
      start();
    }
  private:
    char currentProgram;
    char currentProgramSet;
    Program p[2][4];
    Program lowBatt;
    
    void checkCurrent(){
      if (currentProgram >=4){
        currentProgram = 0;
      }
      if (currentProgramSet >=2){
        currentProgramSet = 0;
      }
    }
    
    void setN(){
      if (currentProgramSet == 0){
        N = 4;
      }else if(currentProgramSet == 1){
        N = 6;
      }
      setPinModes();
    }   
    
    void save(){
      EEPROM.write(1, currentProgramSet);
      EEPROM.write(2, currentProgram);
    }

};

#endif
